/**************************************************************************

  File:        serial.c
  
  Author:      Greg Herlein

  History:     V 0.1   12/96    Inital Version for testing SGI problem 
                 1.0    2/97    First reasonably working version
		 1.1    7/97    Added ReadRawLine(...)

**************************************************************************/

//#define                   DEBUG_RAW_READ_LINE   
//#define                   PRINT_MSG
//#define                   PRINT_TRACE

/* needed constants for UART register access */
#define                   IER                  1   /* interupt enable  */ 
#define                   IID                  2   /* interupt id */ 
#define                   LCR                  3   /* line control */ 
#define                   MCR                  4   /* modem control */ 
#define                   LSR                  5   /* line status   */
#define                   MSR                  6   /* modem status   */

/*
#define                   CD                   0x80
#define                   RI                   0x40
#define                   DSR                  0x20
#define                   CTS                  0x10
#define                   DELTA_CD             0x08
#define                   DELTA_RI             0x04
#define                   DELTA_DSR            0x02
#define                   DELTA_CTS            0x01

#define                   RBR                  0x01
#define                   OVRN_ERR             0x02
#define                   PARITY_ERR           0x04
#define                   FRAME_ERR            0x08
#define                   ALARM_ERR            0x10
#define                   THR_EMPTY            0x20
#define                   TSR_EMPTY            0x40

#define                   DTR                  0x01
#define                   RTS                  0x02
#define                   OCTAGON_XMIT         0x04
#define                   UART_INT             0x08
#define                   UART_TEST            0x10
*/

#define                   SERIAL0              0x03f8    /* standard ports */
#define                   SERIAL1              0x02f8
#define                   SERIAL2              0x03e8
#define                   SERIAL3              0x02e8


/****** INCLUDES *********************************************************/

/* stdlib  */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <string.h>
#include <asm/io.h>

/********************* Global local variables *****************************/

static char                szMsg[80];

/********************  Public Functions **********************************/



int
OpenSerialPort(char* szTTY, int nBaud)
{
  /* opens the applicable serial port

     The nBaud parameter is one of the ones defined in termios.h, ie, B9600

     returns: the value of the open port descriptor, or -1 on error  */


  int                    nFD;
  struct termios         tbuf;

  if((nFD = open(szTTY,O_RDWR))<0) {
    sprintf(szMsg,"Error opening %s: ",szTTY);
    perror(szMsg);
    return -1;
  }

  /* set the termios parameters for raw access to the serial port */
  if(tcgetattr(nFD,&tbuf)<0) return -1;
  tbuf.c_lflag &= ~(ECHO|ICANON|IEXTEN|ISIG);
  tbuf.c_iflag &= ~(BRKINT|ICRNL|INPCK|ISTRIP|IXON);
  tbuf.c_cflag &= ~(CSIZE|PARENB);
  tbuf.c_cflag |= (CS8|CLOCAL|CREAD);
  tbuf.c_oflag &= ~(OPOST);
  
  /* set it so that reads are not blocking, returning 0 if no bytes are
     read, or x bytes that are available */
  tbuf.c_cc[VMIN]=0;
  tbuf.c_cc[VTIME]=0;

  /* set the input and output baud rates */
  cfsetispeed(&tbuf,nBaud);
  cfsetospeed(&tbuf,nBaud);
  
  /* set the port to the attributes defined above */
  if(tcsetattr(nFD,TCSAFLUSH,&tbuf)<0) return -1;

  return nFD;
}



int
CloseSerialPort(int fd)
{
  /* Closes serial communications.
     Returns:  0 on success, -1 on error                     */

  /* close the file descriptor */
  close(fd);

  return 0;
}




int
FlushSerialPort(int fd)
{
  int                  n;

  /* flushes the io buffers 
     Returns:  0 on success, or -1 on error                      */

  n=tcflush(fd,TCIOFLUSH);
  if(n!=0) return -1;

  return 0;
}



int
ReadRawLine(int fd,u_char* buf,int nLen,int nBlock) 
{ 
  /* reads an incoming line into buf of length nLen, using the /n
     byte to mark the stop of a line.  Blocks for no more than 
     nBlock seconds.  Returns on timeout, buffer full, or receipt
     of a complete line.

     Returns:  -1 on error, 0 on timeout, number bytes if OK    */ 

  int                  n,i,nCount=0; 
  int                  nEndFound=0; 
  time_t               tStart;
  time_t               tNow;
  u_char               c=0;
  u_char               cLast=0;
  struct timeval       timeout; 
  fd_set               readset; 

  i=nCount=0;
  tStart=time(NULL);
  while(1) { 
    /* if bytes have arrived, select timeout is 1 sec, 
       (timout period cannot be zero or select will return before
       end of line; 
       otherwise wait for the nBlock seconds for bytes to start arriving */
    timeout.tv_usec=0;
    if(nCount>0)  timeout.tv_sec=1; 
    else timeout.tv_sec=nBlock; 
    FD_ZERO(&readset); 
    FD_SET(fd,&readset); 

    /* use select to wait for bytes arriving...  if they arrive, we read them
       in the block below.  If select times out, we return a timeout */
    n=select(fd+1,&readset,0,0,&timeout); 
    if(n==-1) {
      /* select error detected */
      perror("select error"); 
      fflush(stdout);
      return -1; 
    } else if (n==0) {
      /* timeout */
      return nCount;
    } else {
      /* detect cummulative timeout */
      tNow=time(NULL);
      if(tNow>(tStart+nBlock)) return 0;
      /* do the reading */
      cLast=c;
      n=read(fd,&c,1); 
      if(n<0) { 
	perror("**** Serial Port Read Error: "); 
	return -1; 
      } else if(n==0) {                /* no data ready */
	continue; 
      } else if(n>0) {                 /* data was read */
#ifdef DEBUG_RAW_READ_LINE
	printf("%.2x ",c);
	fflush(stdout);
#endif 
        if(c=='\n') {   
	  nEndFound=1;
	  *(buf+nCount)=c;
	  nCount++;
	  return nCount;
	} else {
	  *(buf+nCount)=c;
	  nCount++;
	  continue;
	}
      } /* endif read was successful */ 
    } /* end if-else-if chain */
  } /* end while */
  return -1; /* ought to never hit this! */ 
} 





int
ReadTermLine(int fd,u_char* buf,int nLen,int nBlock,char cTermChar) 
{ 
  /* reads an incoming line into buf of length nLen, using the cTermChar
     byte to mark the stop of a line.  Blocks for no more than 
     nBlock seconds.  Returns on timeout, buffer full, or receipt
     of a complete line.

     Returns:  -1 on error, 0 on timeout, number bytes if OK    */ 

  int                  n,i,nCount=0; 
  int                  nEndFound=0; 
  time_t               tStart;
  time_t               tNow;
  u_char               c=0;
  u_char               cLast=0;
  struct timeval       timeout; 
  fd_set               readset; 

  i=nCount=0;
  tStart=time(NULL);
  while(1) { 
    /* if bytes have arrived, select timeout is 1 sec, 
       (timout period cannot be zero or select will return before
       end of line; 
       otherwise wait for the nBlock seconds for bytes to start arriving */
    timeout.tv_usec=0;
    if(nCount>0)  timeout.tv_sec=1; 
    else timeout.tv_sec=nBlock; 
    FD_ZERO(&readset); 
    FD_SET(fd,&readset); 

    /* use select to wait for bytes arriving...  if they arrive, we read them
       in the block below.  If select times out, we return a timeout */
    n=select(fd+1,&readset,0,0,&timeout); 
    if(n==-1) {
      /* select error detected */
      perror("select error"); 
      fflush(stdout);
      return -1; 
    } else if (n==0) {
      /* timeout */
      return nCount;
    } else {
      /* detect cummulative timeout */
      tNow=time(NULL);
      if(tNow>(tStart+nBlock)) return 0;
      /* do the reading */
      cLast=c;
      n=read(fd,&c,1); 
      if(n<0) { 
	perror("**** Serial Port Read Error: "); 
	return -1; 
      } else if(n==0) {                /* no data ready */
	continue; 
      } else if(n>0) {                 /* data was read */
#ifdef DEBUG_RAW_READ_LINE
	printf("%.2x ",c);
	fflush(stdout);
#endif 
        if(c==cTermChar) {   
	  nEndFound=1;
	  *(buf+nCount)=c;
	  nCount++;
	  return nCount;
	} else {
	  *(buf+nCount)=c;
	  nCount++;
	  continue;
	}
      } /* endif read was successful */ 
    } /* end if-else-if chain */
  } /* end while */
  return -1; /* ought to never hit this! */ 
} 






int
WriteRawLine(int fd,const u_char* szBuf,int nLen) 
{
  int                    n;
  
#ifdef PRINT_TRACE
    printf("WriteRawLine()\n");
#endif

#ifdef DEBUG_SEND_LINE     
  printf("writing: %s",szBuf);
  printf("SENT: ");
  fflush(stdout);
#endif 

n=write(fd,szBuf,nLen);

#ifdef DEBUG_SEND_LINE     
  printf("\n");
  if(n<0) printf("Error writing to serial port\n");
#endif 

  return n;  
}



int
GetBaudRate(char* szBaud) 
{ 
  int n;

  n=atoi(szBaud);
  switch(n) {
  case 300:
    return B300;
    break;
  case 1200:
    return B1200;
    break;
  case 2400:
    return B2400;
    break;
  case 4800:
    return B4800;
    break;
  case 9600:
    return B9600;
    break;
  case 19200:
    return B19200;
    break;
  case 38400:
    return B38400;
    break;
  case 57600:
    return B57600;
    break;
  case 115200:
    return B115200;
    break;
  default:
    return B57600;
    break;
  } 
}

