/**************************************************************************

  File:        serial_log.c
  
  Author:      Greg Herlein

  This module has several personalities, but all of them connect to
  a serial port and log the data.  The serial byte stream is assumed to
  be line oriented (ie, data records are terminated by a line feed). 

  The program expects the following command line parameters:

  <executable name> port baud filename [term code]

  The port must be a valid serial device name (eg, /dev/cua0)
  The baud must be a valid baud rate supported by the standard UART.
  The filename defines the file that will be logged to; this will be done 
  in append mode.  The optional parameter [term code] defines the termination
  character that the program will use to find the end of a serial string
  coming in the port; this parameter is the decimal value of the ascii
  character that is to be the termination character.

  If compiled as a DAEMON, the program has no controlling terminal 
  and operates totally in background.  If compiled in HEX_MODE, 
  it displays the byte stream in hex on screen; note that the byte 
  stream logged to disk is not changed in any way.  If compiled with 
  TIME_STAMP it prepends unix-time (number of seconds since Jan 1, 1970) 
  and a delta-time (number of seconds since the last record was written)
  comma separated, before the string.  An example of this is below,
  shown with a NEMA string from a gyro:

             874841964,0,$HEHDT,348.4,T

  The beginning of the data stream always begins after the second comma.
  Note that even if in TIME_STAMP the data shown on screen is only the 
  data stream itself.


**************************************************************************/


/****** INCLUDES *********************************************************/

/* stdlib  */
#include <unistd.h>
#include <stdio.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <string.h>
#include <stdlib.h>

/* other */
#include "daemon.h"
#include "serial.h"


/********************* global local variables *****************************/



int main(int argc, char *argv[])
{
  int           fdSerial,n,nBaud;
  char          szBuf[256];
  FILE*         fpLog;
  time_t        t,tLast;
  char          cTerm;

  if(argc<4) {
    printf("usage:  serial_log <port> <baud> <logfile> [term code]\n");
    exit(1);
  }
  nBaud=GetBaudRate(argv[2]);
  fdSerial=OpenSerialPort(argv[1],nBaud);
  if(argc>4) cTerm=(char)atoi(argv[4]);
  else cTerm='\n';

  FlushSerialPort(fdSerial);

  fpLog=fopen(argv[3],"ab");
  if(fpLog==NULL) {
    fprintf(stderr,"Could not open %s (logging file)\n",
	    argv[3]);
    perror("File error: ");
    exit(-1);
  }
  
  printf("Logging serial data stream to %s\n",argv[3]);
  printf("String termination char is ascii %d ",cTerm);
  if(cTerm==10) printf("(LF)\n");
  else if(cTerm==13) printf("(CR)\n");
  else printf("(%c)\n",cTerm);

#ifdef DAEMON
  daemon_init();
#endif

  /* set the inital value of tLast to the starting time (eg, now) */
  tLast=time(&tLast);

  /* start the logging loop */
  while(1) {
    memset(szBuf,0x00,sizeof(szBuf));
    n=ReadTermLine(fdSerial,szBuf,sizeof(szBuf),1,cTerm);
    if(n>0) {

      /* get the present time */
      t=time(&t);

      /* add a line feed char if one is not present */
      n=strlen(szBuf);
      if(szBuf[n-1]!='\n') strcat(szBuf,"\n");
	

      /* log it to the screen if it's not a daemon */
#ifndef DAEMON
  #ifdef HEX_MODE
      n=strlen(szBuf);
      for(i=0;i<n;i++) fprintf(stdout,"%.2x ",szBuf[i]);
      fflush(stdout);
  #endif
  #ifndef HEX_MODE
      fprintf(stdout,"%s",szBuf);
      fflush(stdout);
  #endif
#endif      

      /* log it to disk */      
#ifdef TIME_STAMP
      fprintf(fpLog,"%ld,%ld,%s",t,(t-tLast),szBuf);
      fflush(fpLog);
#endif
#ifndef TIME_STAMP
      fprintf(fpLog,"%s",szBuf);
      fflush(fpLog);
#endif

      /* set tLast to the last time value of a string written */
      tLast=t;

    }
  }

  fclose(fpLog);

  CloseSerialPort(fdSerial);

  return 0;
}

